;======================================================
; Lemmy's Circus Ball
; By Erik557
;======================================================
; Description: A ball that will make Mario bounce if he
; collides with it.
; Uses first extra bit: NO
;======================================================

Tilemap:
       db $4A,$4C,$4A,$4C
YSpeeds:
       db $A0,$D0,$C0,$D0
BounceSpeed:
       db $40,$C0

;=================================
; INIT and MAIN Wrappers
;=================================

print "MAIN ",pc
       PHB
       PHK
       PLB
       JSR CircusBall
       PLB
print "INIT ",pc
       RTL

;========================
; Main routine
;========================

CircusBall:
       JSR Graphics
       LDA $9D
       BNE .Return
       JSL $01802A
       LDA $1588,x
       AND #$04
       BEQ .Interact
       JSL $01ACF9
       AND #$03
       TAY
       LDA YSpeeds,y
       STA $AA,x
.Interact
       JSL $01A7DC
       BCC .Return
       LDA $0E
       CMP #$EB
       BPL .TouchingSide
       LDA $7D
       BMI .TouchingSide
       LDA #$B8
       STA $7D
       LDA #$08
       STA $1DFC
.Return
       RTS
.TouchingSide
       JSR SubHorizPos
       LDA BounceSpeed,y
       STA $7B
       LDA #$08
       STA $1DFC
       RTS

;========================
; Graphics routine
;========================

Graphics:
       JSR GetDrawInfo
       REP #$20
       LDA $00
       STA $0300,y
       SEP #$20
       LDA $14
       LSR #3
       AND #$03
       TAX
       LDA Tilemap,x
       STA $0302,y
       LDX $15E9
       LDA $15F6,x
       ORA $64
       STA $0303,y
       LDY #$02
       TDC
       JSL $01B7B3
       RTS

;========================
; Misc. subroutines
;========================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  GetDrawInfo:            ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$20
       SEC
       SBC $1A
       STA $00
       CLC
       ADC #$0040
       CMP #$0180
       SEP #$20
       LDA $01
       BEQ +
       LDA #$01
+      STA $15A0,x
       TDC
       ROL A
       STA $15C4,x
       BNE .Invalid
       LDA $14D4,x
       XBA
       LDA $1662,x
       AND #$20
       BEQ .CheckOnce
.CheckTwice
       LDA $D8,x
       REP #$21
       ADC #$001C
       SEC
       SBC $1C
       SEP #$20
       LDA $14D4,x
       XBA
       BEQ .CheckOnce
       LDA #$02
.CheckOnce
       STA $186C,x
       LDA $D8,x
       REP #$21
       ADC #$000C
       SEC
       SBC $1C
       SEP #$21
       SBC #$0C
       STA $01
       XBA
       BEQ .OnScreenY
       INC $186C,x
.OnScreenY
       LDY $15EA,x
       RTS

.Invalid
       PLA
       PLA
       RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  SubOffScreen:           ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       
       LDA $15A0,x
       ORA $186C,x
       BEQ .Return2
       LDA $167A,x
       AND #$04
       BNE .Return2
       LDA $5B
       LSR A
       BCS .VerticalLevel
       LDA $D8,x
       ADC #$50
       LDA $14D4,x
       ADC #$00
       CMP #$02
       BCS .EraseSprite
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC .AddTable,y
       SEC
       SBC $1A
       CMP .CmpTable,y
.Common
       SEP #$20
       BCC .Return2
.EraseSprite
       LDA $14C8,x
       CMP #$08
       BCC .KillSprite
       LDY $161A,x
       CPY #$FF
       BEQ .KillSprite
       LDA $1938,y
       AND #$FE
       STA $1938,y
.KillSprite
       STZ $14C8,x
.Return2
       RTS

.VerticalLevel
       LDA $13
       LSR A
       BCS .CheckY
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC.w #$0040
       CMP.w #$0280
       BRA .Common
.CheckY
       LDA $14D4,x
       XBA
       LDA $D8,x
       REP #$20
       SBC $1C
       CLC
       ADC.w #$0070
       CMP.w #$01D0
       BRA .Common

.AddTable
       dw $0040,$0040,$0010,$0070
       dw $0090,$0050,$0080,$FFC0
.CmpTable
       dw $0170,$01E0,$01B0,$01D0
       dw $0230,$01B0,$0220,$0160

;==================
  SubHorizPos:
;==================

       LDY #$00
       LDA $94
       SEC
       SBC $E4,x
       STA $0F
       LDA $95
       SBC $14E0,x
       BPL $01
       INY
       RTS
